/*
 ****************************************************************************
 * Ldap Synchronization Connector provides tools to synchronize
 * electronic identities from a list of data sources including
 * any database with a JDBC connector, another LDAP directory,
 * flat files...
 *
 *                  ==LICENSE NOTICE==
 * 
 * Copyright (c) 2008, LSC Project 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:

 *    * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *     * Neither the name of the LSC Project nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *                  ==LICENSE NOTICE==
 *
 *               (c) 2008 - 2009 LSC Project
 *         Sebastien Bahloul <seb@lsc-project.org>
 *         Thomas Chemineau <thomas@lsc-project.org>
 *         Jonathan Clarke <jon@lsc-project.org>
 *         Remy-Christophe Schermesser <rcs@lsc-project.org>
 ****************************************************************************
 */
package org.lsc.service;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import java.util.Map.Entry;

import javax.naming.NamingException;

import org.apache.log4j.Logger;
import org.lsc.Configuration;
import org.lsc.LscAttributes;
import org.lsc.LscObject;
import org.lsc.objects.Structure;
import org.lsc.objects.flat.fStructure;
import org.lsc.persistence.DaoConfig;

import com.ibatis.sqlmap.client.SqlMapClient;

/**
 * @author Sebastien Bahloul <seb@lsc-project.org>
 */
public final class StructureJdbcService implements ISrcService {

    private Logger LOGGER = Logger.getLogger(StructureJdbcService.class);

    private SqlMapClient sqlMapper;

    private Vector<String> orderedStructures;
    private Map<String, List<String>> structuresChildren;
    private Map<String, fStructure> structures;

    /** Constructor */
    public StructureJdbcService() {
        orderedStructures = new Vector<String>();
        structuresChildren = new HashMap<String, List<String>>();
        structures = new HashMap<String, fStructure>();
		sqlMapper = DaoConfig.getSqlMapClient();
		initialize();
    }

    /* Public Methods */

    private void initialize() {
        List<String> topLevelStructureIds = new ArrayList<String>();
        List originalStructuresList = _getStructuresIdList();
        Iterator oslIter = originalStructuresList.iterator();
        while (oslIter.hasNext()) {
            String structureId = (String) oslIter.next();
            List parent = _getParentStructureId(structureId);
            while(parent.contains(structureId)) {
                parent.remove(structureId);
            }
            if (parent.size() == 0) {
            	if(topLevelStructureIds.size() == 0) {
                    LOGGER.debug("We got the top ! (\"" + structureId + "\")");
                    topLevelStructureIds.add(structureId.toLowerCase());
            	} else {
					LOGGER.warn("There is a big problem with the structure identified by \"" + structureId
					+ "\": no parent structure and the top level has bean already specified by id=" + topLevelStructureIds + " !");
            	}
            } else {
            	String parentId = null;
            	if (parent.size() > 1) {
                	int maxCount = -1;
                	String maxCountTop = null;
                	Iterator parentIter = parent.iterator();
                	while(parentIter.hasNext()) {
                		String parentIdTmp = (String) parentIter.next();
                		int parentIdCount = countSubemployee(parentIdTmp);
                		if(parentIdCount >= maxCount) {
                			maxCount = parentIdCount;
                			maxCountTop = parentIdTmp;
                		}
                	}
                	if(maxCountTop != null) {
                		parentId = maxCountTop;
                		if(!topLevelStructureIds.contains(maxCountTop.toLowerCase())) {
                			topLevelStructureIds.clear();
                			topLevelStructureIds.add(maxCountTop.toLowerCase());
                		}
                	} else {
                        LOGGER.warn("There is a big problem with the structure identified by \"" + structureId
                                + "\": more than one parent !");
                	}
            	} else {
	                parentId = (String) parent.get(0);
            	}
                if (structuresChildren.containsKey(parentId)) {
                    ((List<String>) structuresChildren.get(parentId)).add(structureId);
                } else {
                    List<String> values = new ArrayList<String>();
                    values.add(structureId);
                    structuresChildren.put(parentId, values);
                }
            }
        }
        Iterator siIter = originalStructuresList.iterator();
        while (siIter.hasNext()) {
            String actualStructureId = (String) siIter.next();
            fStructure st = _getStructure(actualStructureId);
            if(topLevelStructureIds.size() > 0) {
                if(topLevelStructureIds.contains(st.getOu().toLowerCase())) {
                    st.setDistinguishName("ou=" + st.getOu() + "," + Configuration.DN_STRUCTURES);
                }
            } else {
            	if(!siIter.hasNext()) {
            		LOGGER.warn("Not able to found one parent, taking the last one ou=" + st.getOu());
            		st.setDistinguishName("ou=" + st.getOu() + "," + Configuration.DN_STRUCTURES);
            	}
            }
            structures.put(actualStructureId, st);
        }
        Iterator<String> topLevelStructureIdsIter = topLevelStructureIds.iterator();
        while(topLevelStructureIdsIter.hasNext()) {
            initializeVector(topLevelStructureIdsIter.next(), new ArrayList<String>());
        }
    }

    private void initializeVector(String structureId, List<String> alreadyDoneStructures) {
    	alreadyDoneStructures.add(structureId);
        orderedStructures.add(structureId);
        List children = (List)structuresChildren.get(structureId);
        if(children != null) {
            Iterator childrenIter = children.iterator();
            while(childrenIter.hasNext()) {
                String childId = (String)childrenIter.next();
                if(!alreadyDoneStructures.contains(childId)) {
                    fStructure childStructure = structures.get(childId);
                    childStructure.setDistinguishName("ou=" + childStructure.getOu() + "," + structures.get(structureId).getDistinguishName());
                    initializeVector(childId, alreadyDoneStructures);
                }
            }
        }
    }
    
    /**
     * Count the nuumber of sub emmployees
     * 
     * @param grhId
     * @return
     */
    public int countSubemployee(String grhId) {
        return _countSubemployee(grhId);
    }

    /**
     * Get the parent id of a specified structure
     * 
     * @param structureId
     * @return
     */
    public Structure getParentStructure(String structureId) {
        List parent = _getParentStructureId(structureId);
        if (parent.size() <= 0) {
            LOGGER.warn("There is a big problem with the structure identified by \"" + structureId
                    + "\": no parent structure !");
        } else if (parent.size() >= 1) {
            LOGGER.warn("There is a big problem with the structure identified by \"" + structureId
                    + "\": more than one parent !");
        } else {
            return (Structure) parent.get(0);
        }
        return null;
    }

    /**
     * Get each child of a specified parent id structure
     * 
     * @param parentId
     * @return
     */
    public List getChildrenStructureId(String structureId) {
        Iterator childrenIdIter = ((List) structures.get(structureId)).iterator();
        List<fStructure> children = new ArrayList<fStructure>();
        while(childrenIdIter.hasNext()) {
            String childrenId = (String) childrenIdIter.next();
            children.add(structures.get(childrenId));            
        }
        return children;
    }

    public fStructure _getStructure(String id) {
		try {
			return (fStructure) sqlMapper.queryForObject("getStructure", id);
		} catch (SQLException e) {
            LOGGER.warn("Unable to found structure with id=" + id + " (" + e + ")", e);
		}
        return null;
   }

    private List _getStructuresIdList() {
        try {
			return (List) sqlMapper.queryForList("getStructuresIdList", null);
		} catch (SQLException e) {
            LOGGER.warn("Unable to return structures list (" + e + ")", e);
		}
		return null;
    }

    private List _getParentStructureId(String structureId) {
        try {
            return (List) sqlMapper.queryForList("getParentStructure", structureId);
		} catch (SQLException e) {
            LOGGER.warn("Unable to return structure parents list (" + e + ")", e);
		} catch (RuntimeException e) {
            LOGGER.warn("Unable to return structure parents list (" + e + ")", e);
		}
		return null;
    }

//    private List _getChildrenStructureId(String structureId) {
//        try {
//            return (List) sqlMapper.queryForList("getChildStructure", structureId);
//		} catch (SQLException e) {
//            LOGGER.warn("Unable to return structure children list (" + e + ")", e);
//		}
//		return null;
//    }

	private int _countSubemployee(String grhId) {
        try {
            return ((Integer) sqlMapper.queryForObject("countSubemployee", grhId)).intValue();
		} catch (SQLException e) {
            LOGGER.warn("Unable to return count sub employes (" + e + ")", e);
		}
		return -1;
	}

	public Map<String, LscAttributes> getListPivots() throws NamingException {
        List<?> ids = orderedStructures.subList(0, orderedStructures.size());
        Iterator<?> idsIter = ids.iterator();
        Map<String,LscAttributes> ret = new HashMap<String, LscAttributes>();
        while(idsIter.hasNext()) {
        	String id = (String) idsIter.next();
            LscAttributes la = new LscAttributes();
           	ret.put(id, la);
        }
        return ret;
	}

	public LscObject getObject(Entry<String, LscAttributes> ids) throws NamingException {
        return structures.get(ids.getKey());
	}
}
